@@ -3,23 +3,21 @@ use warnings;
 
 package Dist::Zilla::Plugin::ReadmeAnyFromPod;
 # ABSTRACT: Automatically convert POD to a README in any format for Dist::Zilla
-$Dist::Zilla::Plugin::ReadmeAnyFromPod::VERSION = '0.142470';
-use Encode qw( encode );
-use IO::Handle;
-use List::Util qw( reduce );
-use Moose::Autobox;
+$Dist::Zilla::Plugin::ReadmeAnyFromPod::VERSION = '0.150250';
+use List::Util 1.33 qw( none first );
 use Moose::Util::TypeConstraints qw(enum);
 use Moose;
 use MooseX::Has::Sugar;
-use Path::Tiny 0.004;
 use Scalar::Util 'blessed';
 
-with 'Dist::Zilla::Role::AfterBuild';
-with 'Dist::Zilla::Role::AfterRelease';
-with 'Dist::Zilla::Role::FileGatherer';
-with 'Dist::Zilla::Role::FileMunger';
-with 'Dist::Zilla::Role::FilePruner';
-with 'Dist::Zilla::Role::FileWatcher';
+with 'Dist::Zilla::Role::AfterBuild',
+    'Dist::Zilla::Role::AfterRelease',
+    'Dist::Zilla::Role::FileGatherer',
+    'Dist::Zilla::Role::FileMunger',
+    'Dist::Zilla::Role::FilePruner',
+    'Dist::Zilla::Role::FileWatcher',
+    'Dist::Zilla::Role::PPI',
+;
 
 # TODO: Should these be separate modules?
 our $_types = {
@@ -49,13 +47,11 @@ our $_types = {
             my $pod = $_[0];
 
             require Pod::Markdown;
+            Pod::Markdown->VERSION('2.000');
             my $parser = Pod::Markdown->new();
-
-            require IO::Scalar;
-            my $input_handle = IO::Scalar->new(\$pod);
-
-            $parser->parse_from_filehandle($input_handle);
-            my $content = $parser->as_markdown();
+            $parser->output_string( \my $content );
+            $parser->parse_characters(1);
+            $parser->parse_string_document($pod);
             return $content;
         },
     },
@@ -135,7 +131,7 @@ sub gather_files {
     my $filename = $self->filename;
     if ( $self->location eq 'build'
          # allow for the file to also exist in the dist
-         and not @{$self->zilla->files->grep( sub { $_->name eq $filename })}
+         and none { $_->name eq $filename } @{ $self->zilla->files }
        ) {
         require Dist::Zilla::File::InMemory;
         my $file = Dist::Zilla::File::InMemory->new({
@@ -159,7 +155,7 @@ sub prune_files {
                 and $_->location eq 'build'
                 and $_->filename eq $self->filename
         } @{$self->zilla->plugins}) {
-        for my $file ($self->zilla->files->flatten) {
+        for my $file (@{ $self->zilla->files }) {
             next unless $file->name eq $self->filename;
             $self->log_debug([ 'pruning %s', $file->name ]);
             $self->zilla->prune_file($file);
@@ -174,7 +170,7 @@ sub munge_files {
 
     if ( $self->location eq 'build' ) {
         my $filename = $self->filename;
-        my $file = $self->zilla->files->grep( sub { $_->name eq $filename } )->head;
+        my $file = first { $_->name eq $filename } @{ $self->zilla->files };
         if ($file) {
             $self->munge_file($file);
         }
@@ -229,32 +225,28 @@ sub _create_readme {
 
         my $content = $self->get_readme_content();
 
-        my $file = $self->zilla->root->file($filename);
-        if (-e $file) {
+        require Path::Tiny;
+        Path::Tiny->VERSION(0.004);
+        my $destination_file = Path::Tiny::path($self->zilla->root, $filename);
+        if (-e $destination_file) {
             $self->log("overriding $filename in root");
         }
         my $encoding = $self->_get_source_encoding();
-        Path::Tiny::path($file)->spew_raw(
+        $destination_file->spew_raw(
             $encoding eq 'raw'
                 ? $content
-                : encode($encoding, $content)
+                : do { require Encode; Encode::encode($encoding, $content) }
         );
     }
 
     return;
 }
 
-sub _file_from_filename {
-    my ($self, $filename) = @_;
-    for my $file ($self->zilla->files->flatten) {
-        return $file if $file->name eq $filename;
-    }
-    die 'no README found (place [ReadmeAnyFromPod] below [Readme] in dist.ini)!';
-}
-
 sub _source_file {
     my ($self) = shift;
-    $self->_file_from_filename($self->source_filename);
+
+    my $filename = $self->source_filename;
+    first { $_->name eq $filename } @{ $self->zilla->files };
 }
 
 # Holds the contents of the source file as of the last time we
@@ -265,18 +257,17 @@ has _last_source_content => (
     default => '',
 );
 
-sub _get_source_content {
-    my ($self) = shift;
-    $self->_last_source_content($self->_source_file->content);
-}
-
 sub _get_source_pod {
     my ($self) = shift;
-    my $source_content = $self->_get_source_content();
 
-    require PPI::Document;
+    my $source_file = $self->_source_file;
+
+    # cache contents before we alter it, for later comparison
+    $self->_last_source_content($source_file->content);
+
+    require PPI::Document;  # for Dist::Zilla::Role::PPI < 5.009
+    my $doc = $self->ppi_document_for_file($source_file);
 
-    my $doc = PPI::Document->new(\$source_content);
     my $pod_elems = $doc->find('PPI::Token::Pod');
     my $pod_content = "";
     if ($pod_elems) {
@@ -284,14 +275,20 @@ sub _get_source_pod {
         $pod_content .= PPI::Token::Pod->merge(@$pod_elems);
     }
 
+    if ((my $encoding = $self->_get_source_encoding) ne 'raw') {
+        require Encode;
+        $pod_content = Encode::decode($encoding, $pod_content);
+    }
+
     return $pod_content;
 }
 
 sub _get_source_encoding {
     my ($self) = shift;
+    my $source_file = $self->_source_file;
     return
-        $self->_source_file->can('encoding')
-            ? $self->_source_file->encoding
+        $source_file->can('encoding')
+            ? $source_file->encoding
             : 'raw';        # Dist::Zilla pre-5.0
 }
 
@@ -339,7 +336,7 @@ Dist::Zilla::Plugin::ReadmeAnyFromPod - Automatically convert POD to a README in
 
 =head1 VERSION
 
-version 0.142470
+version 0.150250
 
 =head1 SYNOPSIS
 
@@ -526,7 +523,7 @@ Karen Etheridge <ether@cpan.org>
 
 =head1 COPYRIGHT AND LICENSE
 
-This software is copyright (c) 2014 by Ryan C. Thompson.
+This software is copyright (c) 2015 by Ryan C. Thompson.
 
 This is free software; you can redistribute it and/or modify it under
 the same terms as the Perl 5 programming language system itself.
